@@ -64,8 +64,10 @@ def _run_simulation(cell, variable_dt=False, atol=0.001):
             t0 = time()
             ti = neuron.h.t
 
-def _run_simulation_with_electrode(cell, electrode, variable_dt=False, atol=0.001,
-                                   to_memory=True, to_file=False, file_name=None):
+def _run_simulation_with_electrode(cell, electrode=None,
+                                   variable_dt=False, atol=0.001,
+                                   to_memory=True, to_file=False,
+                                   file_name=None, electrodecoeffs=None):
     '''
     Running the actual simulation in NEURON.
     electrode argument used to determine coefficient
@@ -80,53 +82,66 @@ def _run_simulation_with_electrode(cell, electrode, variable_dt=False, atol=0.00
 
     # Use electrode object(s) to calculate coefficient matrices for LFP
     # calculations. If electrode is a list, then
-    
-    #put electrode argument in list if needed
-    if type(electrode) == type([]):
-        electrodes = electrode
-    else:
-        electrodes = [electrode]
-    
-    #calculate list of electrodecoeffs, will try temp store of imem, tvec, LFP
-    cellTvec = cell.tvec
-    try:
-        cellImem = cell.imem.copy()
-    except:
-        pass
-    
-    cell.imem = np.eye(cell.totnsegs)
-    cell.tvec = np.arange(cell.totnsegs) * cell.timeres_python
-    electrodecoeffs = []
-    electrodeLFP = []   #list of electrode.LFP objects if they exist
-    restoreLFP = False
-    restoreCellLFP = False
-    for el in electrodes:
-        if hasattr(el, 'LFP'):
-            LFPcopy = el.LFP
-            del el.LFP
-            restoreLFP = True
-        if hasattr(el, 'CellLFP'):
-            CellLFP = el.CellLFP
-            restoreCellLFP = True
-        el.calc_lfp(cell=cell)
-        electrodecoeffs.append(el.LFP.copy())
-        if restoreLFP:
-            del el.LFP
-            el.LFP = LFPcopy
-        else:
-            del el.LFP
-        if restoreCellLFP:
-            el.CellLFP = CellLFP
-        else:
-            if hasattr(el, 'CellLFP'):
-                del el.CellLFP
+    if cell.verbose:
+        print 'precalculating geometry - LFP mapping'
         
-    #putting back variables
-    cell.tvec = cellTvec        
-    try:
-        cell.imem = cellImem
-    except:
-        del cell.imem
+    
+    #assume now that if electrodecoeff != None, we're not using
+    #the electrode object:
+    if electrode != None and electrodecoeffs != None:
+        ERR = "electrode and electrodecoeffs can't both be != None"
+        raise AssertionError, ERR
+    #put electrodecoeff in a list, if it isn't already
+    if electrodecoeffs != None:
+        if type(electrodecoeffs) != type(list):
+            electrodecoeffs = [electrodecoeffs]
+    else:
+        #put electrode argument in list if needed
+        if type(electrode) == type([]):
+            electrodes = electrode
+        else:
+            electrodes = [electrode]
+        
+        #calculate list of electrodecoeffs, will try temp store of imem, tvec, LFP
+        cellTvec = cell.tvec
+        try:
+            cellImem = cell.imem.copy()
+        except:
+            pass
+        
+        cell.imem = np.eye(cell.totnsegs)
+        cell.tvec = np.arange(cell.totnsegs) * cell.timeres_python
+        electrodecoeffs = []
+        electrodeLFP = []   #list of electrode.LFP objects if they exist
+        restoreLFP = False
+        restoreCellLFP = False
+        for el in electrodes:
+            if hasattr(el, 'LFP'):
+                LFPcopy = el.LFP
+                del el.LFP
+                restoreLFP = True
+            if hasattr(el, 'CellLFP'):
+                CellLFP = el.CellLFP
+                restoreCellLFP = True
+            el.calc_lfp(cell=cell)
+            electrodecoeffs.append(el.LFP.copy())
+            if restoreLFP:
+                del el.LFP
+                el.LFP = LFPcopy
+            else:
+                del el.LFP
+            if restoreCellLFP:
+                el.CellLFP = CellLFP
+            else:
+                if hasattr(el, 'CellLFP'):
+                    del el.CellLFP
+            
+        #putting back variables
+        cell.tvec = cellTvec        
+        try:
+            cell.imem = cellImem
+        except:
+            del cell.imem
     
     # Initialize NEURON simulations of cell object    
     neuron.h.dt = cell.timeres_NEURON
@@ -261,3 +276,66 @@ def _run_simulation_with_electrode(cell, electrode, variable_dt=False, atol=0.00
     if to_file:
         el_LFP_file.close()
 
+
+def _collect_geometry_neuron(cell):
+    '''Loop over allseclist to determine area, diam, xyz-start- and
+    endpoints, embed geometry to cell object'''
+    
+    
+    areavec = np.zeros(cell.totnsegs)
+    diamvec = np.zeros(cell.totnsegs)
+    lengthvec = np.zeros(cell.totnsegs)
+    
+    xstartvec = np.zeros(cell.totnsegs)
+    xendvec = np.zeros(cell.totnsegs)
+    ystartvec = np.zeros(cell.totnsegs)
+    yendvec = np.zeros(cell.totnsegs)
+    zstartvec = np.zeros(cell.totnsegs)
+    zendvec = np.zeros(cell.totnsegs)
+    
+    counter = 0
+    
+    #loop over all segments
+    for sec in cell.allseclist:
+        n3d = neuron.h.n3d()
+        if n3d > 0:
+            #length of sections
+            xlength = neuron.h.x3d(n3d - 1) - neuron.h.x3d(0)
+            ylength = neuron.h.y3d(n3d - 1) - neuron.h.y3d(0)
+            zlength = neuron.h.z3d(n3d - 1) - neuron.h.z3d(0)
+            
+            for seg in sec:
+                areavec[counter] = neuron.h.area(seg.x)
+                diamvec[counter] = seg.diam
+                lengthvec[counter] = sec.L/sec.nseg
+                
+                xstartvec[counter] = neuron.h.x3d(0) + \
+                    xlength * (seg.x - 1./2./sec.nseg)
+                xendvec[counter] = neuron.h.x3d(0) + \
+                    xlength * (seg.x + 1./2./sec.nseg)
+                
+                ystartvec[counter] = neuron.h.y3d(0) + \
+                    ylength * (seg.x - 1./2./sec.nseg)
+                yendvec[counter] = neuron.h.y3d(0) + \
+                    ylength * (seg.x + 1./2./sec.nseg)
+                
+                zstartvec[counter] = neuron.h.z3d(0) + \
+                    zlength * (seg.x - 1./2./sec.nseg)
+                zendvec[counter] = neuron.h.z3d(0) + \
+                    zlength * (seg.x + 1./2./sec.nseg)
+                
+                counter += 1
+    
+    
+    cell.xstart = np.array(xstartvec)
+    cell.ystart = np.array(ystartvec)
+    cell.zstart = np.array(zstartvec)
+    
+    cell.xend = np.array(xendvec)
+    cell.yend = np.array(yendvec)
+    cell.zend = np.array(zendvec)
+    
+    cell.area = np.array(areavec)
+    cell.diam = np.array(diamvec)
+    cell.length = np.array(lengthvec)
+
